from decimal import Decimal

from django import forms
from edc_reportable import MILLIMOLES_PER_LITER, ConversionNotHandled, convert_units
from edc_utils.round_up import round_half_away_from_zero

from .constants import GLUCOSE_HIGH_READING


def validate_glucose_as_millimoles_per_liter(
    prefix: str, cleaned_data: dict = None
) -> None | Decimal:
    converted_value = None
    min_val = Decimal("0.00")
    max_val = Decimal("30.00")
    high_value = Decimal(f"{GLUCOSE_HIGH_READING}")
    value = cleaned_data.get(f"{prefix}_value")
    units = cleaned_data.get(f"{prefix}_units")
    if value and units:
        try:
            converted_value = convert_units(
                value, units_from=units, units_to=MILLIMOLES_PER_LITER
            )
        except ConversionNotHandled as e:
            raise forms.ValidationError({f"{prefix}_units": str(e)})
        if (
            not (min_val <= round_half_away_from_zero(converted_value, 2) <= max_val)
            and round_half_away_from_zero(converted_value, 2) != high_value
        ):
            raise forms.ValidationError(
                {
                    f"{prefix}_value": (
                        f"This value is out-of-range. Got {converted_value} mmol/L"
                    )
                }
            )
    return converted_value



from __future__ import annotations

from dateutil.relativedelta import relativedelta
from edc_constants.constants import YES

from ..utils import validate_glucose_as_millimoles_per_liter

INVALID_GLUCOSE_DATE = "INVALID_GLUCOSE_DATE"


class GlucoseFormValidatorMixin:
    prefix: str | None = None

    def validate_glucose_test(self, prefix: str | None = None) -> None:
        prefix = prefix or self.prefix or "glucose"
        self.applicable_if(
            YES, field=f"{prefix}_performed", field_applicable=f"{prefix}_fasting"
        )
        self.required_if(
            YES, field=f"{prefix}_fasting", field_required=f"{prefix}_fasting_duration_str"
        )
        self.required_if(YES, field=f"{prefix}_performed", field_required=f"{prefix}_date")
        self.required_if(YES, field=f"{prefix}_performed", field_required=f"{prefix}_value")
        validate_glucose_as_millimoles_per_liter(prefix, self.cleaned_data)
        self.required_if(
            YES, field=f"{prefix}_performed", field_required=f"{prefix}_quantifier"
        )
        self.required_if(YES, field=f"{prefix}_performed", field_required=f"{prefix}_units")

    def validate_test_date_within_max_months(
        self, date_fld: str, max_months: int | None = None
    ):
        max_months = max_months or 6
        if self.cleaned_data.get(date_fld) and self.cleaned_data.get("report_datetime"):
            try:
                dt = self.cleaned_data.get(date_fld).date()
            except AttributeError:
                dt = self.cleaned_data.get(date_fld)
            report_datetime = self.cleaned_data.get("report_datetime").date()
            rdelta = relativedelta(report_datetime, dt)
            months = rdelta.months + (12 * rdelta.years)
            if months >= max_months or months < 0:
                if months < 0:
                    msg = "Invalid. Cannot be a future date."
                else:
                    msg = (
                        f"Invalid. Must be within the last {max_months} months. "
                        f"Got {abs(months)}m ago."
                    )
                self.raise_validation_error({date_fld: msg}, INVALID_GLUCOSE_DATE)
